import { world, ItemStack, ItemEnchantableComponent, EnchantmentTypes, system} from '@minecraft/server';
import { ModalFormData } from '@minecraft/server-ui';
import { MailBoxes } from './utils/mail.js';
import { Boxes } from './utils/box.js';

const data = new MailBoxes("endxMailboxes");
const dataBox = new Boxes("endxBoxes");

let validIds = [
	"mb:acacia_mailbox",
	"mb:birch_mailbox",
	"mb:cherry_mailbox",
	"mb:crimson_mailbox",
	"mb:dark_oak_mailbox",
	"mb:jungle_mailbox",
	"mb:mangrove_mailbox",
	"mb:oak_mailbox",
	"mb:spruce_mailbox",
	"mb:warped_mailbox"
];
world.afterEvents.playerPlaceBlock.subscribe((e) => {
	const player = e.player;
	const block = e.block;
	
	if (!validIds.includes(block.typeId)) return;
	CreateMenu(block, player);
});

function validateMailboxName(name, player, x, y, z) {
	if (name === '') {
		player.sendMessage({ translate: 'form.mailbox_whitout_name'});
		player.playSound("random.break");
		player.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`);
		return false;
	}
	if (data.mailboxExists(name)) {
		player.sendMessage({ translate: 'form.mailbox_exists'});
		player.playSound("random.break");
		player.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`);
		return false;
	}
	return true;
}

function CreateMenu(block, player) {
	const { x, y, z } = block;
	const coordinates = { x, y, z };
	const dimension = player.dimension.id;
	const CreateForm = new ModalFormData().title({translate: 'form.title.create_mailbox'}).textField({translate: 'form.text.create_mailbox_name'}, {translate: 'form.text.create_mailbox_name'})
	CreateForm.show(player).then(formData => {
		if(formData.canceled){
			player.sendMessage({ translate: 'form.mailbox_not_created'});
			player.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`);
			player.playSound('random.break');
			return;
		}
		const [name] = formData.formValues;
		const mailboxName = `${player.name} - ${name}`;
		if (!validateMailboxName(mailboxName, player, x, y, z)) {
			return;
		}
		player.playSound('random.levelup');
		let containerMailbox = player.dimension.spawnEntity("mb:mailbox_container", { x:x+0.5, y:y+0.5, z:z+0.5});
		containerMailbox.nameTag = "mb.mailbox";
		data.createMailbox(mailboxName, player.name, coordinates, dimension);
		player.sendMessage({ translate: 'form.mailbox_created', with: [`${name}`]});
	})
	.catch((error) => {
		player.sendMessage({translate: 'form.error_show_form'});
		player.runCommandAsync(`setblock ${x} ${y} ${z} air destroy`);
		player.playSound('random.break');
		const allMailboxes = data.getAllMailboxes();
		const mailboxIndex = allMailboxes.findIndex(mailbox => JSON.stringify(mailbox.coordinates) === JSON.stringify(coordinates));
		if (mailboxIndex !== -1) {
			data.removeMailbox(allMailboxes[mailboxIndex].name);
		}
		return -1;
	});
}


world.beforeEvents.playerBreakBlock.subscribe((e) => {
	const player = e.player;
	const block = e.block;
	if (!validIds.includes(block.typeId)) return;
	const { x, y, z } = block;
	const coordinates = { x, y, z };
	const allMailboxes = data.getAllMailboxes();
	const mailboxIndex = allMailboxes.findIndex(mailbox => JSON.stringify(mailbox.coordinates) === JSON.stringify(coordinates));
	
	if (mailboxIndex !== -1) {
		const mailboxPlayerOwner = allMailboxes[mailboxIndex].owner;
		if (mailboxPlayerOwner !== player.name) {
			e.cancel = true;
			return;
		}
	
		data.removeMailbox(allMailboxes[mailboxIndex].name);
	}
});
world.afterEvents.playerInteractWithEntity.subscribe((e) => {
	const player = e.player;
	const item = e.itemStack;
	const target = e.target;
	if(!item) return;
	if(!(item.typeId === "mb:box_item" && target.typeId === "mb:postbox_container")) return;
	SendMenu(player, target);
});
function returnBox(player){
	const returnItem = new ItemStack("mb:box_item", 1);
	player.playSound('random.break');
	player.dimension.spawnItem(returnItem, player.location);
}
function SendMenu(player, target) {
	const listMailboxes = data.getMailboxesByDimension(player.dimension.id);
	if (listMailboxes.length === 0) {
		player.sendMessage({translate: 'form.no_mailboxes_dimension'});
		returnBox(player);
		return;
	}
	const sendItemsInventory = target.getComponent("inventory").container;
	let inventoryItems = [];
	let hasBox = false;

	for(let slot = 0; slot < sendItemsInventory.size; slot++){
		let item = sendItemsInventory.getItem(slot);
		if(item){
			inventoryItems.push(item);
			if(item.typeId === "mb:box_full"){
				hasBox = true;
			}
		}
	}
	if(inventoryItems.length === 0){
		player.sendMessage({ translate: 'form.empty_package'});
		returnBox(player);
		return;
	}
	if(hasBox){
		player.sendMessage({ translate: 'form.has_box'});
		returnBox(player);
		return;
	}
	listMailboxes.sort((a, b) => a.name.localeCompare(b.name));

	const sendItemsForm = new ModalFormData()
	.title({translate: 'form.title.send_box'})
	.dropdown({ translate: 'form.selection.send_box'}, listMailboxes.map(mailbox => mailbox.name), 0);

	sendItemsForm.show(player).then(formData => {
		if(formData.canceled){
			player.sendMessage({ translate: 'form.shipment_cancelled'});
			returnBox(player);
			return;
		}
		const [selection] = formData.formValues;
		const selectedMailbox = listMailboxes[selection];
		const { x, y, z } = selectedMailbox.coordinates;
		const customItem = new ItemStack("mb:box_full", 1);
		
		let loreList = [];
		let itemList = [];
		let boxId = dataBox.getUniqueBoxId();
		loreList.push(`§bid: ${boxId}`);
		for(let slot = 0; slot < sendItemsInventory.size; slot++){
			let itemData = sendItemsInventory.getItem(slot);
			if(!itemData) continue;
			let id = itemData.typeId || '';
			let name = itemData.nameTag || '';
			let amount = itemData.amount || 0;
			let durability = itemData.hasComponent("durability") ? itemData.getComponent("durability").damage : 0;
			let lore = itemData.getLore().length > 0 ? itemData.getLore() : [];
			let enchantmentsList = itemData.hasComponent(ItemEnchantableComponent.componentId) ? itemData.getComponent(ItemEnchantableComponent.componentId).getEnchantments() : [];
			let newItem = dataBox.setItem(id, name, amount, durability, lore, enchantmentsList);

			let itemName = id.split(':')[1];
			itemName = itemName.split('_').map(word => word.charAt(0).toUpperCase() + word.slice(1)).join(' ');
			itemName = `§7${itemName} x${amount}`;
			loreList.push(itemName);
			itemList.push(newItem);
		}
		dataBox.setBox(boxId, itemList);
		customItem.setLore(loreList);
		player.runCommandAsync(`tickingarea add circle ${x} ${y} ${z} 1`);
		target.triggerEvent("mb:despawn");
		system.runTimeout(() =>{
			let item2 = player.dimension.spawnItem(customItem, {x:x+0.5, y:y+0.5, z:z+0.5});
			system.runTimeout(() =>{
				player.runCommandAsync(`tickingarea remove ${x} ${y} ${z}`);
				player.sendMessage({ translate: 'form.box_sent', with: [`${selectedMailbox.name}`]});
				player.playSound('random.levelup');
				return;
			},5);
		},10);
	})
	.catch((error) => {
		player.sendMessage({ translate: 'form.problem_shipment'});
		returnBox(player);
		return -1;
	});
}


function createItemFromData(itemData) {
	if (!itemData.id || itemData.amount === 0) return null;
	const itemFromBox = new ItemStack(itemData.id, parseInt(itemData.amount));
	if (itemData.name) itemFromBox.nameTag = itemData.name;
	if (itemData.durability > 0) itemFromBox.getComponent("durability").damage = itemData.durability;
	if (itemData.lore) itemFromBox.setLore(itemData.lore);
	if (itemData.enchantmentsList) {
		for (let enchantment of itemData.enchantmentsList) {
			itemFromBox.getComponent(ItemEnchantableComponent.componentId).addEnchantment({type:EnchantmentTypes.get(enchantment.type.id), level:enchantment.level});
		}
	}
	return itemFromBox;
}

world.afterEvents.itemUse.subscribe((e) => {
	const player = e.source;
	const item = e.itemStack;
	/*if (item.typeId === "minecraft:iron_sword") {
		let vartemp = world.getDynamicPropertyTotalByteCount();
		player.sendMessage(`BYTES: ${vartemp}`);
	}*/
	if (item.typeId === "mb:box_full" && item.getLore().length > 0) {
		const lore = item.getLore()[0];
		const match = lore.match(/§bid:\s*(\d+)/);
		const boxId = match ? parseInt(match[1]) : null;
		if (!boxId) {
			player.sendMessage({ translate: 'form.box_already_used'});
			return;
		}
		const box = dataBox.getBoxById(boxId);
		if (!box) {
			player.sendMessage({ translate: 'form.box_already_used'});
			return;
		}
		for (let i = 0; i < box.itemList.length; i++) {
			let itemFromBox = createItemFromData(box.itemList[i]);
			if (itemFromBox) {
				player.dimension.spawnItem(itemFromBox, player.location);
			}
		}
		dataBox.openBox(boxId);
	}
});
